/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your application specific code will go here
 */
define(['ojs/ojcore', 'knockout', 'ojs/ojmodule-element-utils', 'toastr', 'ojs/ojknockout', 'ojs/ojmodule-element', 'ojs/ojrouter', 'ojs/ojarraytabledatasource', 'ojs/ojmoduleanimations'],
  function(oj, ko, moduleUtils, toastr) {
     function ControllerViewModel() {
      var self = this;

      // Global observables:
      self.userId = ko.observable();
      self.userRole = ko.observable();
      self.serviceId = ko.observable();
      self.baseUrl = ko.observable("https://appstest.evry.no:8282/utv/ntnu/");

      // Save the theme so we can perform platform specific navigational animations
      var platform = oj.ThemeUtils.getThemeTargetPlatform();

      // Router setup
      self.router = oj.Router.rootInstance;
      self.previousState = ko.observable("");

      self.router.configure({
       'dashboard': {label: 'Dashboard'},
       'frontpage': {label: 'Forside'},
       'login': {label: '', isDefault: true},
       'profile': {label: 'Min Side'},
       'forgotPassword': {label: ''},
       'showId': {label: 'ID-kort'},
       'myInterests': {label: 'Mine interesser'},
       'addInterests': {label: 'Legg til interesser'},
       'editPerson/{type}/{data}': {label: 'Oppdater dine opplysninger'},
       'settings': {label: 'Innstillinger'},
       'registerHoursList': {label: 'Timeliste'},
       'editRegisteredHours': {label: 'Registrer timer'},
       'unavailability': {label: 'Utilgjengelighet'},
       'editUnavailability/{tgh_id}': {label: 'Rediger avtale'},
       'newUnavailability': {label: 'Ny avtale'},
       'editVacation/{fri_id}': {label: 'Rediger ferie'},
       'newVacation': {label: 'Ny ferie'},
       'services': {label: 'Tjenester'},
       'serviceInfo/{tje_id}': {label: 'Informasjon'},
       'calendar': {label: 'Kalender'},
       'editPassword': {label: 'Endre passord'},
       'adminCalendar': {label: 'Administratorkalender'},
       'adminFrontpage': {label: 'Forside'},
       'camera': {label: 'Ta profilbilde'},
       'adminList/{list_id}/{extra}': {label: 'Liste'},
       'adminVolunteerProfile/{user_id}': {label: 'Frivilligprofil'},
       'adminCreateService': {label: 'Opprett ny tjeneste'}
      });

      oj.Router.defaults['urlAdapter'] = new oj.Router.urlParamAdapter();
      // Callback function that can return different animations based on application logic.
      function switcherCallback(context) {
        if (platform === 'android')
          return 'fade';
        return null;
      };

      self.moduleConfig = ko.observable({'view':[], 'viewModel':null});

      self.loadModule = function() {
        ko.computed(function() {
          var name = self.router.moduleConfig.name();
          var viewPath = 'views/' + name + '.html';
          var modelPath = 'viewModels/' + name;
          var masterPromise = Promise.all([
            moduleUtils.createView({'viewPath':viewPath}),
            moduleUtils.createViewModel({'viewModelPath':modelPath})
          ]);
          masterPromise.then(
            function(values){
              self.moduleConfig({'view':values[0],'viewModel':values[1]});
            }
          );
        });
      }
      self.moduleAnimation = oj.ModuleAnimations.switcher(switcherCallback);

      // Navigation setup
      var navData = [
      {name: 'Dashboard', id: 'dashboard',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-chart-icon-24'},
      {name: 'Forside', id: 'frontpage',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-fire-icon-24'},
      {name: 'Logg inn', id: 'login',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-people-icon-24'},
      {name: 'Min Side', id: 'profile',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-person-icon-24'},
      {name: 'ID-kort', id: 'showId',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-info-icon-24'},
       {name: 'Mine interesser', id: 'myInterests',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-info-icon-24'},
       {name: 'Legg til interesser', id: 'addInterests',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-info-icon-24'},
       {name: 'Registrer timer', id: 'registerHoursList',
       iconClass: 'oj-navigationlist-item-icon demo-icon-font-24 demo-info-icon-24'}
      ];

      // Toastr-functions:
      toastr.options = {
        "closeButton": true,
        "debug": false,
        "newestOnTop": false,
        "progressBar": false,
        "positionClass": "toast-bottom-full-width",
        "preventDuplicates": false,
        "onclick": null,
        "showDuration": "300",
        "hideDuration": "1000",
        "timeOut": "3000",
        "extendedTimeOut": "1000",
        "showEasing": "swing",
        "hideEasing": "linear",
        "showMethod": "fadeIn",
        "hideMethod": "fadeOut"
      };

      self.showSaveSuccess = function(message) {
        toastr.success(message, "Lagret!",  {timeOut: 1000});
      }

      self.showSaveError = function() {
        toastr.error("Vennligst prøv igjen senere. Hvis problemet vedvarer, ta kontakt med administrator.", "Klarte ikke å lagre");
      }

      self.showDeleteSuccess = function(message) {
        toastr.success(message, "Slettet");
      }

      self.showGeneralError = function() {
        toastr.error("Vennligst prøv igjen senere. Hvis problemet vedvarer, ta kontakt med administrator.", "Noe gikk galt")
      }

      //self.navDataSource = new oj.ArrayTableDataSource(navData, {idAttribute: 'id'});

      var stateArray = new Array();

      // Header Setup
      self.getHeaderModel = function() {
        this.pageTitle = self.router.currentState().label;

        this.backClicked = function() {
          // Prevents from going back if previous screen if on frontpage
          if (self.router.currentState().id != "frontpage") {
            // Goes back to front page if on profile page, regardless of previous page
            if (self.router.currentState().id == "profile") {
              self.router.go("frontpage");
              for (var i = stateArray.length; i > 1; i--) { // Clears the array
                stateArray.pop();
              }
            }
            // Checks if it should skip a view (used after someone saves an edit)
            else if (self.router.currentState().id == stateArray[stateArray.length - 3]) {
              self.router.go(stateArray[stateArray.length - 4]);
              stateArray.pop(); stateArray.pop();
              stateArray.pop(); stateArray.pop();
            }
            else { // None of the checks match, goes back normally
              self.router.go(stateArray[stateArray.length - 2]);
              stateArray.pop(); stateArray.pop();
            }
          }
        }

        this.myPageClicked = function() {
          // Goes to profile page as long as user is not on login or profile page
          if (self.router.currentState().id != "login"
          || self.router.currentState().id != "profile") self.router.go("profile");
        }
        this.transitionCompleted = function() {
          stateArray.push(self.router.currentState().id); // Adds new state to the array
          console.log(stateArray);
          self.adjustContentPadding();
        }
      };

      // Method for adjusting the content area top/bottom paddings to avoid overlap with any fixed regions.
      // This method should be called whenever your fixed region height may change.  The application
      // can also adjust content paddings with css classes if the fixed region height is not changing between
      // views.
      self.adjustContentPadding = function () {
        var topElem = document.getElementsByClassName('oj-applayout-fixed-top')[0];
        var contentElem = document.getElementsByClassName('oj-applayout-content')[0];
        var bottomElem = document.getElementsByClassName('oj-applayout-fixed-bottom')[0];

        if (topElem) {
          contentElem.style.paddingTop = topElem.offsetHeight+'px';
        }
        if (bottomElem) {
          contentElem.style.paddingBottom = bottomElem.offsetHeight+'px';
        }
        // Add oj-complete marker class to signal that the content area can be unhidden.
        // See the override.css file to see when the content area is hidden.
        contentElem.classList.add('oj-complete');
      }
    }

    return new ControllerViewModel();
  }
);
