/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */

 // Based on answer from: https://stackoverflow.com/questions/9206914/how-to-filter-multidimensional-javascript-array
function filterByProperty(array, value){
  var filtered = [];
  for(var i = 0; i < array.length; i++){
      var obj = array[i];
      // Lowercase item, category and search-value so the search is not case-sensitive:
      var str = obj.item.toLowerCase();
      if (obj.category != undefined) {        // Check if it exist a category
        str += obj.category.toLowerCase();
      }
      // Filtering based on part of string: https://stackoverflow.com/questions/1789945/how-to-check-whether-a-string-contains-a-substring-in-javascript
      if (str.indexOf(value.toLowerCase()) != -1) {
        filtered.push(obj)
      }
  }    

  return filtered;

}

/*
 * Your addInterests ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojarraydataprovider', 'ojs/ojlistview', 'ojs/ojbutton', 'ojs/ojinputtext'],
 function(oj, ko, $, app, moduleUtils, ArrayDataProvider) {
  
    function AddInterestsViewModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      self.rootViewModel;           // Initiate variable.
      self.data = [];
      self.selectedItems = ko.observableArray([]); // observable bound to selection option to monitor current selections

      self.getNewInterests = function(url) {
        $.ajax({
          dataType: "json",
          url: url,
          success: function(interests) {
            $.each(interests.items, function() {
              self.data.push({
                  id: this.id, 
                  item: this.navn,
                  description: this.beskrivelse,
                  status: this.status,
                  category: this.kategori
              });
          });

          // Get if array is bigger than 10 and if it is, only show 10 in for loop below:
          var length = 10;
          if (self.data.length < length) {
            length = self.data.length;
          }

          // Need to copy items onto a new array manually with a for-loop to be sure the data-array is not changed when the observable array filteredItems is changed.
          // With a for-loop I can even change how many items to show at the beginning.
          for(var i = 0; i < length; i++) {
            self.itemsToShow.push(self.data[i]);
          }
          self.filteredItems(self.itemsToShow);   // Show first all items in list.
      },
          error: function() { self.rootViewModel.showGeneralError(); }
        });
      }
     
     self.itemsToShow = []

      self.searchString = ko.observable();
      self.filteredItems = ko.observableArray(self.itemsToShow);   // Show first all items in list.

      // If searchString is changed (user is typing in search), call this function. Found here: https://stackoverflow.com/questions/12489835/setting-a-call-back-function-for-observable-objects
      self.searchString.subscribe(function () {
        var filtered = filterByProperty(self.data, self.searchString())

        self.filteredItems.removeAll();               // Remove what is in array now.

        // Get if array is bigger than 10 and if it is, only show 10 in for loop below:
        var length = 10;
        if (filtered < length) {
          length = filtered.length;
        }
        
        // Add search-results to listview (observable array wichh update listview automatically).
        for (var i = 0; i < length; i++) {
          self.filteredItems.push(filtered[i]);
        }
      });

    self.dataProvider = new ArrayDataProvider(self.filteredItems, {'keyAttributes': 'id'});


    // Add every interest with the same number of api-calls:
    self.addButtonClicked = function(event){
      for (i = 0; i < self.selectedItems().length; i++) {
        let itemId = self.selectedItems()[i];
        var body = {
          tjeneste_id: itemId,
          bruker_id: self.rootViewModel.userId(),
        };
  
        $.ajax({
          type: "POST",
          url: self.rootViewModel.baseUrl() + "frivillig/" + self.rootViewModel.userId() + "/interesser/finn_nye",
          // The key needs to match your method's input parameter (case-sensitive).
          data: JSON.stringify(body),
          contentType: "application/json; charset=utf-8",
          dataType: "json",
          success: function(data){
            // Remove interest from list and show message:
            self.filteredItems.remove(function(item)
            {
                return (item.id == itemId);
            });
            self.rootViewModel.showSaveSuccess();
          },
          failure: function(errMsg) {
              self.rootViewModel.showSaveError();           // Show error.
          }
      });
      }
    }

    // Search items:
    self.search = function() {
      var filtered = filterByProperty(self.data, self.searchString())

        self.filteredItems.removeAll();               // Remove what is in array now.

        // Get if array is bigger than 10 and if it is, only show 10 in for loop below:
        var length = 10;
        if (filtered < length) {
          length = filtered.length;
        }

        // Add search-results to listview (observable array wichh update listview automatically).
        for (var i = 0; i < length; i++) {
          self.filteredItems.push(filtered[i]);
        }
    }

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here. 
       * This method might be called multiple times - after the View is created 
       * and inserted into the DOM and after the View is reconnected 
       * after being disconnected.
       */
      self.connected = function() {
        self.rootViewModel = ko.dataFor(document.getElementById('globalBody'));     // Refresh every time we go here.
        
        //If userId is undefined, go back to login-page:
        if (self.rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        
        // Zero out data before refresh/adding data to list:
        self.data = [];
        self.itemsToShow = [];
        self.getNewInterests(self.rootViewModel.baseUrl() + "frivillig/" + self.rootViewModel.userId() + "/interesser/finn_nye")
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    
    return new AddInterestsViewModel();
  }
);
