/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your calendar ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojarraydataprovider', 'moment', 'fullcalendar', 'ojs/ojpopup', 'ojs/ojrouter', 'ojs/ojbutton', 'ojs/ojlabel', 'ojs/ojselectcombobox', 'ojs/ojlistview', 'ojs/ojnavigationlist'],
 function(oj, ko, $, app, moduleUtils, ArrayDataProvider, moment) {

    function AdminCalendarViewModel() {
      var self = this;

      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      var serviceIdList = [];
      var animationCounter = 0;
      console.log("User ID in calendar: " + userId)

      this.title = userId;
      self.popupTitle = ko.observable();
      self.popupPlace = ko.observable();
      self.popupContent = ko.observable();
      self.popupVolunteerInfo = ko.observable();
      self.popupParticipantsInfo = ko.observable();

      self.selectedItems = ko.observableArray([]);

      self.events = ko.observableArray();
      self.calendarFilter = ko.observable("all");           // Contains what we shall show in calendar

      self.selectedState = ko.observable("calendarTab");
      self.org_nr = ko.observable(null);

      self.eventsUnhandled = [];                                    // Events that are shown in "Tjenester som det haster å gjøre noe med"
      self.datasource = ko.observableArray(self.eventsUnhandled);

      self.dataProvider = new ArrayDataProvider(self.datasource, {'keyAttributes': 'id'});

      var currentEventId = 0;
      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      self.itemSelected = function() {
        oj.Router.rootInstance.go('/serviceInfo/' + self.eventsUnhandled[self.selectedItems()[0]].eventId);
     };

      self.closePopup1 = function (event)
      {
        var popup = document.getElementById('popup1');
        popup.close();
      }.bind(this);

      self.closeFilterPopup = function (event)
      {
        var popup = document.getElementById('filterPopup');
        popup.close();
        self.calendar.fullCalendar('refetchEvents')
      }.bind(this);

      self.seeMorePopup = function (event)
      {
        console.log("Read more-button clicked for event: " + self.popupTitle());
        oj.Router.rootInstance.go('/serviceInfo/' + currentEventId);
      }.bind(this);

      self.getPosition = function ()
      {
        return {'my' : {'horizontal': 'center',
                        'vertical': 'center'
                       },
                'at' : {'horizontal': 'center',
                        'vertical': 'center'
                       },
                'collision' : 'none'};
      }.bind(this);

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        resetCalendar();
        
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      /**
       * Updates view when changing tabs.
       */
      self.changeView = function () {
        if (self.selectedState() == "calendarTab") {
          resetCalendar();
        }
        else {
            document.getElementById("calendarDiv").style.display = "none";
            document.getElementById("listViewDiv").style.display = "block";

            console.log("Changed view to a listview with urgent events")
            // Check if org-number exists before calling for get-unhandled-events, and if it not know of org-number, get that first:
            let getUnhandledEventsWrapper = function(){
              getUnhandledEvents(self.org_nr(), moment().format("DD_MM_YYYY"), moment().add(10, "days").format("DD_MM_YYYY"));
            };
            if (self.org_nr() != null) {
              getUnhandledEventsWrapper();
            }
            else {
              self.getOrgNr(getUnhandledEventsWrapper);
            }
        }
      }


      /**
       * Get org-number and call function.
       */
      self.getOrgNr = function(callback) {
        $.ajax({
          dataType: "json",
          url: "https://appstest.evry.no:8282/utv/ntnu/admin/hentOrganisasjonsNr/" + rootViewModel.userId(),
          success: function(result) {
            self.org_nr(result.org_orgnr);
              callback();
        },
          error: function() { self.rootViewModel.showGeneralError(); }
        });
      }

      /**
      * Set color on events.
      */
      self.colorpicker = function(event) {

        animationCounter++;
        if (animationCounter >= serviceIdList.length) {
        
          var lis = document.getElementById("listViewDiv").getElementsByTagName("li");
          for (var j = 0; j < serviceIdList.length; j++) {
            var element = lis[j];
            for(var i = 0; i < self.eventsUnhandled.length; i++) {
              if (serviceIdList[j] == self.eventsUnhandled[i].eventId) {
                if (self.eventsUnhandled[i].volunteersRegistered < self.eventsUnhandled[i].volunteersWanted && self.eventsUnhandled[i].volunteersRegistered > 0) {
                  element.style.backgroundColor = "yellow";
                }
                else if (self.eventsUnhandled[i].volunteersRegistered <= 0) {
                  element.style.backgroundColor = "red";
                }
              }
            }
          }
        }
      };

      /**
       * Get Events that are not fullbooked by volunteers.
       * @param {*} org_nr 
       * @param {*} start_date 
       * @param {*} end_date 
       */
      function getUnhandledEvents(org_nr, start_date, end_date) {
          self.eventsUnhandled = [];
          serviceIdList = [];
          var currItems = 0;
          let url = "https://appstest.evry.no:8282/utv/ntnu/admin/tjenester/ikke_bemannet/" + org_nr + "/" + start_date + "/" + end_date;
          $.ajax({
            dataType: "json",
            url: url,
            success: function(result) {
              $.each(result.items, function() {
                var color;
                var eventId = this.id;
                var title = this.navn;
                var startTime = $.fullCalendar.moment(this.dato_tid);
                var timeEnd = moment(startTime).add(this.varighet, 'hours');
                var info = "";

                // If place is defined (in variable 'sted'):
                if (this.sted != undefined && this.sted != null) {
                  info = this.sted;
                }

                // If area is defined (in variable 'omrade'):
                if (this.omrade != undefined && this.omrade != null) {
                  info += " (" + this.omrade + ")";
                }

                // if either place or area or both is defined:
                if ((this.sted != undefined && this.sted != null) || (this.omrade != undefined && this.omrade != null)) {
                  info += " - "
                }

                info += startTime.format("DD.MM.YYYY HH:mm") + " - " + timeEnd.format("HH:mm");

                if (this.frivillige_paameldt >= this.ant_frivillige) {
                    color = "green";
                  }
                  else if (this.frivillige_paameldt <= this.ant_frivillige && this.frivillige_paameldt > 0) {
                    color = "yellow";
                  }
                  else if (this.frivillige_paameldt <= 0) {
                    color = "red";
                  }

                self.eventsUnhandled.push({
                    title: title,
                    start: startTime,
                    eventId: eventId,
                    end: $.fullCalendar.moment(startTime).add(this.varighet, 'hours'),
                    description: this.beskrivelse,
                    place: this.sted,
                    color: color,
                    textColor: "black",
                    info: info,
                    volunteersWanted: this.ant_frivillige,
                    volunteersRegistered: this.frivillige_paameldt
                });
                serviceIdList[currItems] = eventId;
                currItems++;

              });

              self.datasource(self.eventsUnhandled);

            },
            error: function() { self.rootViewModel.showGeneralError(); }
          });
      }

      /**
       * Reset the calendar
       */
      function resetCalendar() {
        $(function() {

          document.getElementById("listViewDiv").style.display = "none";
          document.getElementById("calendarDiv").style.display = "block";

          // page is now ready, initialize the calendar...

          self.calendar = $('#calendar').fullCalendar({
            customButtons: {
              filter: {
                text: 'Filtrer kalender',
                id: 'filterButton',
                click: function() {
                  var popup = document.getElementById('filterPopup');
                  popup.position = self.getPosition();
                  popup.open('body');
                }
              }
            },
            aspectRatio: 0.9,
            header: {
              left: 'prev,next today filter',
              center: 'title',
              right: 'month,agendaWeek,agendaDay,listMonth'
            },
            locale: 'nb',
            firstDay: 1,
            dayNamesShort: ['Søn ','Man','Tirs','Ons','Tors','Fre','Lør'],
            dayNames: ['Søndag', 'Mandag', 'Tirsdag', 'Onsdag', 'Torsdag', 'Fredag', 'Lørdag'],
            monthNames: ['Januar', 'Februar', 'Mars', 'April', 'Mai', 'Juni', 'Juli',
            'August', 'September', 'Oktober', 'November', 'Desember'],
            monthNamesShort: ['Jan', 'Feb', 'Mar', 'Apr', 'Mai', 'Jun',
            'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Des'],
            weekNumberTitle: "Uke",
            timeFormat: 'H(:mm)', // uppercase H for 24-hour clock
            timezone: 'Europe/Oslo',
            buttonText: {
              today:    'I dag',
              month:    'Måned',
              week:     'Uke',
              day:      'Dag',
              list:     'Liste'
            },
            eventLimitText: 'flere',
            weekNumbers: true,
            navLinks: true, // can click day/week names to navigate views
            editable: false,
            eventLimit: true, // allow "more" link when too many events
            slotLabelFormat: "H(mm)",
            timezone: "Europe/Oslo",

            events: function(start, end, timezone, callback) {
              self.events([]);         // Initiate/empty events.
              // Get all events the volunteeer is working on
              let getEventsWrapper = function() {
                let url = "";
                // If we shall show all events in this organisation.
                if (self.calendarFilter() == "all") {
                  url = rootViewModel.baseUrl() + 'admin/tjenester/alle/' + self.org_nr() + "/" + start.format('DD_MM_YYYY') + "/" + end.format('DD_MM_YYYY');
                }
                else if (self.calendarFilter() == "urgent") {       // If we want to show urgent messages in calendar (instead of list), will show whole month/week/day.
                  url = rootViewModel.baseUrl() + 'admin/tjenester/ikke_bemannet/' + self.org_nr() + "/" + start.format('DD_MM_YYYY') + "/" + end.format('DD_MM_YYYY');
                }
                getEvents(url, callback, false, start, end, timezone);
              };
              if (self.org_nr() != null) {
                getEventsWrapper();
              }
              else {
                self.getOrgNr(getEventsWrapper);
              }
            },

            eventClick: function(calEvent, jsEvent, view) {
              console.log("ID for event: " + calEvent.eventId);
              currentEventId = calEvent.eventId;
              console.log($(this));
              self.popupTitle(calEvent.title);
              self.popupPlace(calEvent.place);
              self.popupContent(calEvent.description);
              self.popupVolunteerInfo(calEvent.volunteersRegistered + " frivillige påmeldt av ønskede " + calEvent.volunteersWanted);
              let participantsInfo = "Deltagere må forhåndsregistreres: " + calEvent.registration
              if (calEvent.registration == "Ja") {
                participantsInfo += " (" + calEvent.participantsRegistered + " av totalt " + calEvent.participantsMax + ")"
              }
              self.popupParticipantsInfo(participantsInfo);
              var popup = document.getElementById('popup1');
              popup.position = self.getPosition();
              popup.open('body');
            },

            // Special options for each view:
            views: {
              basic: {
                // options apply to basicWeek and basicDay views
              },
              agenda: {
                // options apply to agendaWeek and agendaDay views
              },
              week: {
                // options apply to basicWeek and agendaWeek views
                columnHeaderFormat: "ddd D/M"
              },
              day: {
                // options apply to basicDay and agendaDay views
                columnHeaderFormat: "dddd D/M"
              }
            }


          })

        });
      }

      /**
       * Get events
       * @param {*} url 
       * @param {*} callback 
       * @param {*} doCallback 
       * @param {*} start 
       * @param {*} end 
       * @param {*} timezone 
       */
      function getEvents(url, callback, doCallback, start=null, end=null, timezone=null) {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        $.ajax({
          dataType: "json",
          url: url,
          success: function(result) {
            console.log("Result: ")
            console.log(result)
            $.each(result.items, function() {
              var color;
              var eventId = this.id;
              var title = this.navn + " (" + this.frivillige_paameldt + "/" + this.ant_frivillige + ")";
              var startTime = $.fullCalendar.moment(this.dato_tid);

              // Set color based on number of volunteers:
              if (this.frivillige_paameldt >= this.ant_frivillige) {
                color = "green";
              }
              else if (this.frivillige_paameldt <= this.ant_frivillige && this.frivillige_paameldt > 0) {
                color = "yellow";
              }
              else if (this.frivillige_paameldt <= 0) {
                color = "red";
              }

              // Set participants_registrated to 0 if return null:
              if (this.deltagere_paameldt != null) {
                participants_registrated = this.deltagere_paameldt
              }
              else {
                participants_registrated = 0;
              }

              self.events().push({
                  title: title,
                  start: startTime,
                  eventId: eventId,
                  end: $.fullCalendar.moment(startTime).add(this.varighet, 'hours'),
                  description: this.beskrivelse,
                  place: this.sted,
                  color: color,
                  textColor: "black",
                  volunteersWanted: this.ant_frivillige,
                  volunteersRegistered: this.frivillige_paameldt,
                  registration: this.paamelding,
                  participantsMax: this.ant_deltagere,
                  participantsRegistered: participants_registrated
              });

            });

            // Finished adding all, tell fullcalendar that we are finished:
            callback(self.events());

        },
          error: function() { self.rootViewModel.showGeneralError(); }
        });   
      }

    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new AdminCalendarViewModel();
  }
);
