/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your about ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils',
        'ojs/ojselectcombobox', 'ojs/ojinputnumber', 'ojs/ojcheckboxset', 'ojs/ojselectcombobox',
        'ojs/ojbutton', 'ojs/ojarraydataprovider', 'ojs/ojknockout', 'ojs/ojinputtext', 'ojs/ojlabel', "ojs/ojdatetimepicker"],
 function(oj, ko, $, app, moduleUtils) {

    function AdminCreateServiceViewModel() {
      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));

      var self = this;
      var userId = rootViewModel.userId();
      var orgId="";
      this.selectedServiceType = ko.observable();
      this.serviceTypes = ko.observableArray([]);

      this.serviceTypeList = new oj.ArrayDataProvider(this.serviceTypes, {keyAttributes: 'value'});

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        var checkbox = document.getElementById('options');
        checkbox.classList.add('oj-checkboxset-no-chrome');
        document.getElementById("serviceType").setProperty('translations.filterFurther', 'Skriv for å søke opp flere alternativer');

        var today = new Date();
        var dateElement = document.getElementById("date");
        dateElement.translations = {
          cancel: 'Avbryt',
          currentText: 'I dag',
          done: 'Ferdig'
        };
        dateElement.value = oj.IntlConverterUtils.dateToLocalIso(today);


        var timeElement = document.getElementById("duration");
        var hours = today.getHours()+1 < 10 ? "0" + (today.getHours() + 1) : today.getHours() + 1;
        var minutes = today.getMinutes() < 10 ? "0" + today.getMinutes() : today.getMinutes();
        console.log("2018-04-01T" + hours + ":" + minutes + ":00.000Z");
        timeElement.value = "2018-04-01T" + hours + ":" + minutes + ":00.000Z";

        getOrgId();
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      self.saveData = function() {
        var regist = document.getElementById("registration").hidden == true ? "Nei" : "Ja";
        var startTime = new Date(document.getElementById("duration").value);
        var endTime = new Date(document.getElementById("date").value);
        var duration = (endTime.getHours() - startTime.getHours());
        var dateFormat = endTime.getDate() + "." + endTime.getMonth() + "." + endTime.getFullYear() + " " + endTime.getHours() + ":"  + endTime.getMinutes();
        console.log("XD: " + duration);
        var body = {
          orgnr: orgId,
          tjt_id: document.getElementById("serviceType").value,
          tittel: document.getElementById("title").value,
          paamelding: regist,
          ant_frivillige: document.getElementById("participants").value,
          dato_tid: dateFormat,
          varighet: duration,
          timer: duration,
          adresse: document.getElementById("place").value,
          beskrivelse: document.getElementById("comment").value,
          alder: document.getElementById("minAge").value
        }

        console.log(body);

        $.ajax({
          type: "POST",
          url: rootViewModel.baseUrl() + "tjenester/nyTjeneste",
          // The key needs to match your method's input parameter (case-sensitive).
          data: JSON.stringify(body),
          contentType: "application/json; charset=utf-8",
          dataType: "json",
          success: function(data) {
            console.log(data);
          },
          error: function(data) {
            console.log(data);
          }
        });
      };

      self.newOptions = function(event) {
        toggleVisibility(event.explicitOriginalTarget.value);
      };

      self.logg = function(event) {
        console.log(event);
        var placeElement = document.getElementById("place");
        placeElement.focus();
      }

      function getOrgId() {
        $.ajax({
          type: "GET",
          url: rootViewModel.baseUrl() + "admin/hentOrganisasjonsNr/9999",
          success: function(data){
            console.log(data);
            orgId = data.org_orgnr;
            getServiceTypes();
          },
          error: function() {
            console.log("error");
          }
        });
      }

      function getServiceTypes() {
        $.ajax({
          type: "GET",
          url: rootViewModel.baseUrl() + "tjenester/hentTjenestetype/" + orgId,
          success: function(data){
            console.log(data);
            for (var i = 0; i < data.count; i++) {
              self.serviceTypes.push({"value": data.items[i].tjt_id, "label": data.items[i].tjt_navn});
              if (i == 0) self.selectedServiceType = data.items[i].tjt_id;
            }
            document.getElementById("serviceType").value = self.selectedServiceType;
            console.log("TEST" + self.selectedServiceType);
          }
        });
      }

      function toggleVisibility(option) {
        var elementId = "";
        switch (option) {
          case "ageRest": elementId = "age"; break;
          case "regist": elementId = "partips"; break;
        }
        var currentElement = document.getElementById(elementId);
        currentElement.hidden = (currentElement.hidden ? false : true);
      }
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new AdminCreateServiceViewModel();
  }
);
