/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your frontpage ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojmasonrylayout', 'ojs/ojrouter', 'ojs/ojbutton'],
 function(oj, ko, $, app, moduleUtils) {

    function FrontpageViewModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */

       /**
        * Want to go to calendar.
        */
      self.calendarButtonClicked = function() {
        console.log("Button for calendar has been pressed");
        oj.Router.rootInstance.go('adminCalendar');
      }

      /**
       * Want to see list of users.
       */
      self.showUserListButtonClicked = function() {
        console.log("Button for seeing a list of users");
        oj.Router.rootInstance.go('adminList/volunteers');
      }

      /**
       * Want to see on events.
       */
      self.eventsButtonClicked = function() {
        console.log("Button for events has been pressed");
        oj.Router.rootInstance.go('services');
      }

      /**
       * Register times you are unavailable.
       */
      self.cameraButtonClicked = function() {
        oj.Router.rootInstance.go('camera');
      }

      /**
       * Show id.
       */
      self.showIdButtonClicked = function() {
        console.log("Button for showing id has been pressed");
        oj.Router.rootInstance.go('showId');
      }

      self.connected = function() {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          if (rootViewModel.userRole() != 3 || rootViewMode.userRole() != 4) {
            oj.Router.rootInstance.go('frontpage');
          }
          oj.Router.rootInstance.go('login');
          return
        }
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new FrontpageViewModel();
  }
);
