/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */

 // Based on answer from: https://stackoverflow.com/questions/9206914/how-to-filter-multidimensional-javascript-array
 function filterByProperty(array, value){
    var filtered = [];
    for(var i = 0; i < array.length; i++){
        var obj = array[i];
        // Lowercase item, category and search-value so the search is not case-sensitive:
        var str = obj.item.toLowerCase();
        // Filtering based on part of string: https://stackoverflow.com/questions/1789945/how-to-check-whether-a-string-contains-a-substring-in-javascript
        // If value is undefined accept all:
        if (value == undefined || str.indexOf(value.toLowerCase()) != -1) {
          filtered.push(obj)
        }
    }    
  
    return filtered;
  
  }


  
  /*
   * Your admin list ViewModel code goes here
   */
  define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojarraydataprovider', 'moment', 'ojs/ojlistview', 'ojs/ojbutton', 'ojs/ojinputtext'],
   function(oj, ko, $, app, moduleUtils, ArrayDataProvider, moment) {
    
      function AdminListViewModel() {
        var self = this;
  
        // Header Config
        self.headerConfig = ko.observable({'view':[], 'viewModel':null});
        moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
          self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
        })
  
        self.rootViewModel;           // Initiate variable.
        self.data = [];
        self.selectedItems = ko.observableArray([]); // observable bound to selection option to monitor current selections

        self.listLength = ko.observable(10);        // Length of list (number of items to show)

        /**
         * Get items to show from url
         */
        self.getNewItems = function(url) {
          $.ajax({
            dataType: "json",
            url: url,
            success: function(items) {
              $.each(items.items, function() {
                let item = "";
                let subItem = "";
                if (self.listType() == "volunteers") {
                  item = this.fornavn + " " + this.etternavn;
                }
                else if (self.listType() == "serviceVolunteers") {
                  item = this.navn + " " + this.etternavn;
                }
                else if (self.listType() == "volunteerServices") {
                  let date = moment(this.dato_tid);
                  let endTime = moment(date).add(this.original_varighet, 'hours');
                  let endTimeFinal = moment(date).add(this.endret_varighet, 'hours');
                  item = this.navn;
                  subItem = date.format("DD.MM.YYYY: HH:mm") + " - " + endTime.format("HH:mm")

                  // If a time is changed, write new time + evt. explanation.
                  if (this.timer_endret == "JA") {
                    // Convert duration to a time:
                    subItem += " (Ny sluttid: " +endDateFinal.format('HH:mm');

                    if (this.merknad != undefined && this.merknad != null) {
                      subItem += " - Merknad: " + this.merknad;
                    }

                    subItem += ")";
                  }
                }
                else if (self.listType() == "volunteerInterests") {
                  item = this.navn;
                }
                self.data.push({
                    id: this.id, 
                    item: item,
                    subItem: subItem
                });
            });

            self.filteredItems(self.getItemsToShow(self.data));   // Show first all items in list.
          },
            error: function() { self.rootViewModel.showGeneralError(); }
          });
        }
       
       self.itemsToShow = []
  
        self.searchString = ko.observable();
        self.filteredItems = ko.observableArray(self.itemsToShow);   // Show first all items in list.
  
        // If searchString is changed (user is typing in search), call this function. Found here: https://stackoverflow.com/questions/12489835/setting-a-call-back-function-for-observable-objects
        self.searchString.subscribe(function () {
          var filtered = filterByProperty(self.data, self.searchString())
  
          self.filteredItems.removeAll();               // Remove what is in array now.
  
         self.filteredItems(self.getItemsToShow(filtered));
        });
  
      self.dataProvider = new ArrayDataProvider(self.filteredItems, {'keyAttributes': 'id'});
  
  
      // Go to more info (the volunteers profile):
      self.listItemClicked = function(event){
        if (self.listType() == "volunteers" || self.listType() == "serviceVolunteers") {
          oj.Router.rootInstance.go('adminVolunteerProfile/' + self.selectedItems()[0]);
        }
        /*else if (self.listType() == "volunteerServices") {
          oj.Router.rootInstance.go('serviceInfo/' + self.selectedItems()[0]);    // TODO: Change to admin-services when possible.
        }*/
      }
  
      // Search items:
      self.search = function() {
        var filtered = filterByProperty(self.data, self.searchString())
  
          self.filteredItems.removeAll();               // Remove what is in array now.
  
          self.filteredItems(self.getItemsToShow(filtered));
      }

      /**
       * Get back an array of elements to show from the inputted array.
       */
      self.getItemsToShow = function(inputArray) {
        // Get if array is bigger than defined length and if it is, only show that amount of items in for loop below:
        let length = self.listLength();
        if (inputArray.length < length) {
          length = inputArray.length;
          document.getElementById('loadMoreButton').disabled = true;      // Disable if all loaded.
        }
        else {
            document.getElementById('loadMoreButton').disabled = false;      // Enable if more possible.
        }

        let itemsToShow = [];
  
        // Need to copy items onto a new array manually with a for-loop to be sure the data-array is not changed when the observable array filteredItems is changed.
        // With a for-loop I can even change how many items to show at the beginning.
        for(var i = 0; i < length; i++) {
          itemsToShow.push(inputArray[i]);
        }
        return itemsToShow
    }

    /**
     * Load more elements in list.
     */
    self.loadMoreButtonClicked = function() {
        document.getElementById('loadMoreButton').disabled = true;      // Disable until loaded more.
        self.listLength(self.listLength() + 10);                        // Will load 10 more from now on.
        self.search();                                                  // Update list.
      }


  
        // Below are a set of the ViewModel methods invoked by the oj-module component.
        // Please reference the oj-module jsDoc for additional information.
  
        /**
         * Optional ViewModel method invoked after the View is inserted into the
         * document DOM.  The application can put logic that requires the DOM being
         * attached here. 
         * This method might be called multiple times - after the View is created 
         * and inserted into the DOM and after the View is reconnected 
         * after being disconnected.
         */
        self.connected = function() {
          self.rootViewModel = ko.dataFor(document.getElementById('globalBody'));     // Refresh every time we go here.
          
          //If userId is undefined, go back to login-page:
          if (self.rootViewModel.userId() == undefined) {
            oj.Router.rootInstance.go('login');
            return
          }

          self.filteredItems([]);         // Remove any items that might still be there.
          self.searchString("");          // Remove any information in searchstring.

          self.listLength(10);

          // Get what we shall get a list on:
        var urlString = window.location.href;
        var url = new URL(urlString);
        var values = url.searchParams.get("root").split("/");
        this.tjeId = values[1]; // Id of the service is sent through the url

        self.listType = ko.observable(values[1]);

        // If it exist a extra value like a service id, add it here:
        if (values.length > 2) {
            self.listTypeExtra = ko.observable(values[2]);
        }
        else {
            self.listTypeExtra = ko.observable(null);       // If it do not exist, add is as null
        }
          
          let apiEndpoint = "admin/liste/" + self.rootViewModel.userId() + "/";    // Saves the api-endpoint to use

          // Go to correct api-endpoint:
          if (self.listType() == "volunteers") {       // Get all users
              apiEndpoint += "frivillig";
          }
          else if (self.listType() == "serviceVolunteers") {       // Get all users on a specific service
              apiEndpoint += "tjenester/" + self.listTypeExtra();
          }
          else if (self.listType() == "volunteerServices") {       // Get all users on a specific service
            apiEndpoint += "frivillig/" + self.listTypeExtra() + "/tjenester";
          }
          else if (self.listType() == "volunteerInterests") {       // Get all users on a specific service
            apiEndpoint += "frivillig/" + self.listTypeExtra() + "/interesser";
          }

          // Zero out data before refresh/adding data to list:
          self.data = [];
          self.itemsToShow = [];
          self.getNewItems(self.rootViewModel.baseUrl() + apiEndpoint);
        };
  
        /**
         * Optional ViewModel method invoked after the View is disconnected from the DOM.
         */
        self.disconnected = function() {
          // Implement if needed
        };
  
        /**
         * Optional ViewModel method invoked after transition to the new View is complete.
         * That includes any possible animation between the old and the new View.
         */
        self.transitionCompleted = function() {
          // Implement if needed
        };
      }
  
      /*
       * Returns a constructor for the ViewModel so that the ViewModel is constructed
       * each time the view is displayed.  Return an instance of the ViewModel if
       * only one instance of the ViewModel is needed.
       */
      
      return new AdminListViewModel();
    }
  );
  