/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your calendar ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'moment', 'fullcalendar', 'ojs/ojpopup', 'ojs/ojrouter', 'ojs/ojbutton', 'ojs/ojlabel', 'ojs/ojselectcombobox'],
 function(oj, ko, $, app, moduleUtils) {

    function CalendarViewModel() {
      var self = this;

      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      console.log("User ID in calendar: " + userId)

      this.title = userId;
      self.popupTitle = ko.observable();
      self.popupPlace = ko.observable();
      self.popupContent = ko.observable();

      self.events = ko.observableArray();
      self.calendarFilter = ko.observable("added");           // Contains what we shall show in calendar

      var currentEventId = 0;
      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      self.closePopup1 = function (event)
      {
        var popup = document.getElementById('popup1');
        popup.close();
      }.bind(this);

      self.closeFilterPopup = function (event)
      {
        var popup = document.getElementById('filterPopup');
        popup.close();
        self.calendar.fullCalendar('refetchEvents')
      }.bind(this);

      self.seeMorePopup = function (event)
      {
        console.log("Read more-button clicked for event: " + self.popupTitle());
        oj.Router.rootInstance.go('/serviceInfo/' + currentEventId);
      }.bind(this);

      self.getPosition = function ()
      {
        return {'my' : {'horizontal': 'center',
                        'vertical': 'center'
                       },
                'at' : {'horizontal': 'center',
                        'vertical': 'center'
                       },
                'collision' : 'none'};
      }.bind(this);

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        resetCalendar();
        
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      /**
       * Reset the calendar
       */
      function resetCalendar() {
        $(function() {

          // page is now ready, initialize the calendar...

          self.calendar = $('#calendar').fullCalendar({
            customButtons: {
              filter: {
                text: 'Filtrer kalender',
                id: 'filterButton',
                click: function() {
                  var popup = document.getElementById('filterPopup');
                  popup.position = self.getPosition();
                  popup.open('body');
                }
              }
            },
            aspectRatio: 0.9,
            header: {
              left: 'prev,next today filter',
              center: 'title',
              right: 'month,agendaWeek,agendaDay,listMonth'
            },
            locale: 'nb',
            firstDay: 1,
            dayNamesShort: ['Søn ','Man','Tirs','Ons','Tors','Fre','Lør'],
            dayNames: ['Søndag', 'Mandag', 'Tirsdag', 'Onsdag', 'Torsdag', 'Fredag', 'Lørdag'],
            monthNames: ['Januar', 'Februar', 'Mars', 'April', 'Mai', 'Juni', 'Juli',
            'August', 'September', 'Oktober', 'November', 'Desember'],
            monthNamesShort: ['Jan', 'Feb', 'Mar', 'Apr', 'Mai', 'Jun',
            'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Des'],
            weekNumberTitle: "Uke",
            timeFormat: 'H(:mm)', // uppercase H for 24-hour clock
            timezone: 'Europe/Oslo',
            buttonText: {
              today:    'I dag',
              month:    'Måned',
              week:     'Uke',
              day:      'Dag',
              list:     'Liste'
            },
            eventLimitText: 'flere',
            weekNumbers: true,
            navLinks: true, // can click day/week names to navigate views
            editable: false,
            eventLimit: true, // allow "more" link when too many events
            slotLabelFormat: "H(mm)",
            timezone: "Europe/Oslo",

            events: function(start, end, timezone, callback) {
              self.events([]);         // Initiate/empty events.
              // Get all events the volunteeer is working on
              let volunteerUrl = rootViewModel.baseUrl() + 'tjenester/oppdrag/' + rootViewModel.userId() + "/" + start.format('DD_MM_YYYY') + "/" + end.format('DD_MM_YYYY');
              getEvents(volunteerUrl, callback, false, start, end, timezone);
            },

            eventClick: function(calEvent, jsEvent, view) {
              console.log("ID for event: " + calEvent.eventId);
              currentEventId = calEvent.eventId;
              console.log($(this));
              self.popupTitle(calEvent.title);
              self.popupPlace(calEvent.place);
              self.popupContent(calEvent.description);
              var popup = document.getElementById('popup1');
              popup.position = self.getPosition();
              popup.open('body');
            },

            // Special options for each view:
            views: {
              basic: {
                // options apply to basicWeek and basicDay views
              },
              agenda: {
                // options apply to agendaWeek and agendaDay views
              },
              week: {
                // options apply to basicWeek and agendaWeek views
                columnHeaderFormat: "ddd D/M"
              },
              day: {
                // options apply to basicDay and agendaDay views
                columnHeaderFormat: "dddd D/M"
              }
            }


          })

        });
      }

      /**
       * Get events
       * @param {*} url 
       * @param {*} callback 
       * @param {*} doCallback 
       * @param {*} start 
       * @param {*} end 
       * @param {*} timezone 
       */
      function getEvents(url, callback, doCallback, start=null, end=null, timezone=null) {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        $.ajax({
          dataType: "json",
          url: url,
          success: function(result) {
            $.each(result.items, function() {
              var color;
              var eventId = this.id;
              var title = this.navn;
              var startTime = $.fullCalendar.moment(this.dato_tid);

              if (this.frivillige_paameldt >= this.ant_frivillige) {
                color = "green";
              }
              else if (this.frivillige_paameldt <= this.ant_frivillige && this.frivillige_paameldt > 0) {
                color = "yellow";
              }
              else if (this.frivillige_paameldt <= 0) {
                color = "red";
              }

              if (this.endret_varighet != null && this.endret_varighet != this.varighet) {
                title += " (Ny sluttid: " + $.fullCalendar.moment(startTime).add(this.endret_varighet, 'hours').format("H.mm") + ")";
              }
              self.events().push({
                  title: title,
                  start: startTime,
                  eventId: eventId,
                  end: $.fullCalendar.moment(startTime).add(this.varighet, 'hours'),
                  description: this.beskrivelse,
                  place: this.sted,
                  color: color,
                  textColor: "black"
              });

            });

            // If finished adding all, tell fullcalendar that we are finished:
            if (doCallback || self.calendarFilter() == "added") {
              callback(self.events());
            }
            else if (self.calendarFilter() == "addedAndInterested") {     // Add interested.
              getEvents(rootViewModel.baseUrl() + 'tjenester/oppdrag/interesser/' + rootViewModel.userId() + "/" + start.format('DD_MM_YYYY') + "/" + end.format('DD_MM_YYYY'), callback, true);
            }
            else if (self.calendarFilter() == "all") {                    // Add all.
              getEvents(rootViewModel.baseUrl() + 'tjenester/oppdrag/alle/' + rootViewModel.userId() + "/" + start.format('DD_MM_YYYY') + "/" + end.format('DD_MM_YYYY'), callback, true);
            }

        },
          error: function() { self.rootViewModel.showGeneralError(); }
        });
      }

    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new CalendarViewModel();
  }
);
