/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your frontpage ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojmasonrylayout', 'ojs/ojrouter', 'ojs/ojbutton'],
 function(oj, ko, $, app, moduleUtils) {

    function FrontpageViewModel() {
      var self = this;

      var rawURI = "";  // Raw image URI
      var imgData = ""; // Image URI with added base64 syntax

      self.pictures = ko.observableArray();

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      /**
        Set settings for the oj-camera plugin
      */
       function setOptions(srcType) {
        var options = {
          // Some common settings are 20, 50, and 100
          quality: 50,
          destinationType: Camera.DestinationType.FILE_URI,
          // In this app, dynamically set the picture source, Camera or photo gallery
          sourceType: srcType,
          encodingType: Camera.EncodingType.JPEG,
          mediaType: Camera.MediaType.PICTURE,
          allowEdit: true,
          correctOrientation: true  //Corrects Android orientation quirks
        }
        return options;
      }

      /**
       * Take picture and store it as base64 in rawURI and imgData
       */
      self.openCamera = function() {

        var srcType = Camera.PictureSourceType.CAMERA;
        var options = setOptions(srcType);

        navigator.camera.getPicture(
          function (imageURI) {
            rawURI = imageURI;
            imgData = 'data:image/png;base64,' + imageURI;
            let img = document.getElementById("displayImage");
            let txt = document.getElementById("displayURI");
            img.src = imgData;
          },
          function  (err) {
            //fill out
          },
          {
            quality: 50,
            destinationType: Camera.DestinationType.FILE_URI
          }
        );
      }

      /**
      Converts the base64 image data into BLOB data

       Function taken from https://stackoverflow.com/questions/16245767/creating-a-blob-from-a-base64-string-in-javascript
      */
      self.b64toBlob = (b64Data, contentType='', sliceSize=512) => {
        const byteCharacters = atob(b64Data);
        const byteArrays = [];

        for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
          const slice = byteCharacters.slice(offset, offset + sliceSize);

          const byteNumbers = new Array(slice.length);
          for (let i = 0; i < slice.length; i++) {
            byteNumbers[i] = slice.charCodeAt(i);
          }

          const byteArray = new Uint8Array(byteNumbers);
          byteArrays.push(byteArray);
        }

        const blob = new Blob(byteArrays, {type: contentType});
        return blob;
      }

      self.newClick = function() {
        self.openCamera();
        document.getElementById("displayImage").src = "";
      }

      /*
        Function for when the user wants to upload a captured picture
      */
      self.keepClick = function() {
        var contentType = "image/png";
        var fileType = contentType.split("/");  // Extracts the filetype from the contentType variable
        var target = 7601; // Hardcoded to user "Test2" for testing purposes
        var blob = self.b64toBlob(rawURI, contentType); // Converts the URI into BLOB
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));

        // Data to be sent with the request as parameters
        var fd = new FormData();
        fd.append('bruker_id', target);
        fd.append('orgNr', 9999999999);
        fd.append('filNavn', target + "." + fileType[1]);
        fd.append('mimeType', contentType);
        fd.append('imgUri', blob);  // CURRENTLY NONFUNCTIONAL - BLOB data is not stored in the database

        $.ajax({
          type: 'POST',
          url: rootViewModel.baseUrl() + 'admin/nyttbilde',
          data: fd,
          processData: false,
          contentType: false
        }).done(function(data) {
          console.log(data);
        });
      }

      self.connected = function() {
        self.openCamera();

        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          if (rootViewModel.userRole() != 3 || rootViewMode.userRole() != 4) {
            oj.Router.rootInstance.go('frontpage');
          }
          oj.Router.rootInstance.go('login');
          return
        }
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new FrontpageViewModel();
  }
);
