/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your incidents ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojbutton', 'ojs/ojknockout', 'ojs/ojinputtext', 'ojs/ojlabel'],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;

    function EditPersonViewModel(viewParams) {
      var self = this;
      var rootViewModel;

      oj.Router.sync();

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        var urlString = window.location.href;
        var url = new URL(urlString);
        values = url.searchParams.get("root").split("/");

        // Datatype and value is sent through the url
        var type = values[1];
        var data = values[2];

        // The input field is filled with the existing data
        self.type = ko.observable(type);
        self.data = ko.observable(data);

        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Function for saving the new information back to the database
      self.saveData = function() {
        var userId = rootViewModel.userId._latestValue;
        var data = {};
        var url = rootViewModel.baseUrl() + "oppdater/";
        console.log(self.type._latestValue);

        // Switch is finding out what datatype is changed
        switch (self.type._latestValue) {
          case "Navn": url += "navn/" + userId;
              var name = nameSplit(document.getElementById("userInputEdit").value);
              data.fornavn = name[0];
              data.etternavn = name[1];
              break;
          case "Email": url += "epost/" + userId;
              data.epost = document.getElementById("userInputEdit").value;
              break;
          case "Mobil": url += "mobil/" + userId;
              data.mobil = document.getElementById("userInputEdit").value;
              break;
          case "Tlf": url += "tlf/" + userId;
              data.telefon = document.getElementById("userInputEdit").value;
              break;
        }

        var xhttp = new XMLHttpRequest();
        xhttp.open("PUT", url, true);
        xhttp.setRequestHeader('Content-type', 'application/json');
        var json = JSON.stringify(data);
        xhttp.send(json);

        // When data is successfully saved, go back to the profile page
        xhttp.onreadystatechange = function() {
          if (xhttp.readyState == XMLHttpRequest.DONE) {
            rootViewModel.showSaveSuccess();
            router.go('/profile');
          }
        }
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new EditPersonViewModel();
  }
);

// Function for splitting a simple string into two parts based on number of words
function nameSplit(name) {
  var fullName = name.split(" ");
  console.log(fullName);
  var result = [];
  result[0] = "";  result[1] = "";
  var nameSize = fullName.length;
  for (var i = 0; i < (nameSize)/2; i++) {
    result[0] += fullName[i] + " ";
  }
  for (var i = Math.ceil(nameSize/2); i < nameSize; i++) {
    result[1] += fullName[i] + " ";
  }
  return result;
}
