/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */

// Found here: https://stackoverflow.com/questions/8043026/how-to-format-numbers-by-prepending-0-to-single-digit-numbers/50757763
function formatNumber(number) {
  return ("0" + number).slice(-2);
}

// Based at answer here: https://stackoverflow.com/questions/1050720/adding-hours-to-javascript-date-object
function addHours(date, h) {
  return new Date(date.getTime() + (h*3600000));
}

// Based at answer here: https://stackoverflow.com/questions/1050720/adding-hours-to-javascript-date-object
function addMinutes(date, m) {
  return new Date(date.getTime() + (m*60*1000));
}

/*
 * Your editRegisteredHoursViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'moment', 'ojs/ojrouter', 'ojs/ojselectcombobox', 'ojs/ojinputtext', 'ojs/ojlabel', 'ojs/ojbutton'],
 function(oj, ko, $, app, moduleUtils, moment) {

    var router = oj.Router.rootInstance;

    function editRegisteredHoursViewModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      self.serviceId = 0;
      self.userId = 0;
      self.data = {};
      self.datasource = ko.observable(self.data);
      self.registeredHours = ko.observable();
      self.newDurationNotice = ko.observable();
      self.rootViewModel = ko.dataFor(document.getElementById('globalBody'));

      self.getServiceInfo = function(serviceId) {
          $.ajax({
            dataType: "json",
            url: self.rootViewModel.baseUrl() + "tjenester/frivilligavtale/" + serviceId,
            success: function(service) {
              // Get data:
              let date = moment(service.dato_tid);
              let endDate = moment(date).add(service.original_varighet, 'hours');
              let contact = ""
              if (service.kontaktperson != null) {
                contact = "Oppdragsgiver: " + service.kontaktperson
              }
              self.data = {
                id: service.id,
                name: service.navn,
                description: service.beskrivelse,
                contact: contact,
                date: date.format('DD.MM.YYYY'),
                startTime: date.format('HH:mm'),
                endTime: endDate.format('HH:mm'),
                durationChanged: service.timer_endret,      // Is "JA" if has changed.
                changedDuration: service.endret_varighet,   // Duration both if it is changed or not (but contains the latest value).
                changeNotice: service.endret_merknad
            };
            console.log(self.data)
            self.datasource(self.data);
            self.registeredHours(service.endret_varighet.toString())
            self.newDurationNotice(service.endret_merknad)
        },
            error: function() { self.rootViewModel.showGeneralError(); }
        });
      }

      self.datasource(self.data);

      self.saveNewDuration = function() {
        console.log("Saves new duration as " + self.registeredHours() + " and notice as " + self.newDurationNotice());

        var body = {
          avtale_id: self.data.id,
          bruker_id: self.userId,
          endret_varighet: self.registeredHours(),
          endret_merknad: self.newDurationNotice()
        };
        console.log(body)
        $.ajax({
          type: "POST",
          url: self.rootViewModel.baseUrl() + "tjenester/frivilligavtale/" + self.data.id,
          // The key needs to match your method's input parameter (case-sensitive).
          data: JSON.stringify(body),
          contentType: "application/json; charset=utf-8",
          dataType: "json",
          success: function(data){
            self.showSaveSuccess("Endringer er nå lagret");
            router.go("/registerHoursList");
          },
          failure: function(errMsg) {
              self.showSaveError();
          }
      });
      }

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        console.log("Id gotten from observable: " + rootViewModel.serviceId());

        //If id is undefined, go back to list:
        if (rootViewModel.serviceId() == undefined) {
          oj.Router.rootInstance.go('registerHoursList');
          return
        }

        self.serviceId = rootViewModel.serviceId();
        self.userId = rootViewModel.userId();

        self.data = {};
        self.getServiceInfo(self.serviceId);
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new editRegisteredHoursViewModel();
  }
);
