/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojselectcombobox', 'ojs/ojinputnumber', 'ojs/ojselectcombobox', 'ojs/ojbutton', 'ojs/ojknockout', 'ojs/ojinputtext', 'ojs/ojlabel', "ojs/ojdatetimepicker"],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;
    function EditUnavailabilityViewModel() {
      var self = this;

      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();

      self.timeFrom = ko.observable();
      self.timeTo = ko.observable();

      this.title = userId;
      this.tghId = "";
      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        var urlString = window.location.href;
        var url = new URL(urlString);
        var values = url.searchParams.get("root").split("/");
        this.tghId = values[1]; // Id of the unavailability is sent through the url
        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFast/" + values[1];

        //Get all data from the specific unavailability
        var xhttp = new XMLHttpRequest();
        xhttp.open("GET", url, true);
        xhttp.send();
        var selector = document.getElementById("basicSelect");
        var clockTo = document.getElementById("clockTo");
        var clockFrom = document.getElementById("clockFrom");
        var other = document.getElementById("other");

        //  Fill in the fields with the current data
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
             var JSONData = JSON.parse(this.responseText);
             selector.value = JSONData.tgh_dag.toString();
             other.value = JSONData.tgh_merknad.toString();

             // Formats and injects the stored time into the input fields
             var fr = document.getElementById("clockFrom");
             fr.value = (Math.floor((JSONData.tgh_kl_fra / 100)) >= 10 ? "T" : "T0") + Math.floor(JSONData.tgh_kl_fra / 100)
             + ((JSONData.tgh_kl_fra % 100) >= 10 ? ":" : ":0") + JSONData.tgh_kl_fra % 100 + ":00";
             console.log(fr);

             var to = document.getElementById("clockTo");
             to.value = ((Math.floor(JSONData.tgh_kl_til / 100)) >= 10 ? "T" : "T0") + Math.floor(JSONData.tgh_kl_til / 100)
             + ((JSONData.tgh_kl_til % 100) >= 10 ? ":" : ":0") + JSONData.tgh_kl_til % 100 + ":00";
             console.log(to);

/*
             self.timeFrom().setHours(5);
             var fr = self.timeFrom();
             //fr.setHours(3);
             //var fr = oj.IntlConverterUtils.dateToLocalIso(new Date(2000, 0, 1, JSONData.tgh_kl_fra / 100, JSONData.tgh_kl_fra % 100));
             //self.timeFrom
             //self.timeTo = oj.IntlConverterUtils.dateToLocalIso(new Date(2000, 0, 1, JSONData.tgh_kl_til / 100, JSONData.tgh_kl_til % 100));
             //var fr = new Date("T10:30:00");
             //alert(fr);
             //self.timeFrom = fr.toISOString();
             //self.timeFrom.setHours(JSONData.tgh_kl_fra / 100);
             alert(fr + " og " + self.timeFrom().toISOString());
             self.timeTo = "T10:30:00";
             alert(self.timeFrom() + " og " + self.timeTo());
             /*clockTo.value = JSONData.tgh_kl_til;
             clockFrom.value = JSONData.tgh_kl_fra;*/

          }
        };
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Function for saving updated information to the database
      self.saveData = function() {
        var data = {};
        data.tgh_dag = document.getElementById("basicSelect").value;
        data.tgh_kl_fra = timeToDecimal(self.timeFrom());
        data.tgh_kl_til = timeToDecimal(self.timeTo());
        data.tgh_merknad = document.getElementById("other").value;

        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFast/" + self.tghId;
        console.log(url);
        var xhttp = new XMLHttpRequest();
        xhttp.open("PUT", url, true);
        xhttp.setRequestHeader('Content-type', 'application/json');
        var json = JSON.stringify(data);
        console.log(json);
        xhttp.send(json);
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            rootViewModel.showSaveSuccess("Endringer er nå lagret");
            router.go('/unavailability');
          }
        };
      };

      // Function for deleting post from database
      self.deletePost = function() {
        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFast/" + self.tghId;
        console.log(url);
        var xhttp = new XMLHttpRequest();
        xhttp.open("DELETE", url, true);
        xhttp.send();
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            rootViewModel.showDeleteSuccess("Du er nå tilgjengelig på dette tidspunktet");
            router.go('/unavailability');
          }
        };
      };
    }

    // Converts the ISOtime to decimal with four digits
    // Example: T14:52:00 -> 1452
    function timeToDecimal(time) {
      var hoursMinutes = time.split(/[T.:]/);
      var hours = hoursMinutes[1];
      var minutes = hoursMinutes[2];
      return hours + minutes;
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new EditUnavailabilityViewModel();
  }
);
