/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojselectcombobox',
    'ojs/ojinputnumber', 'ojs/ojdatetimepicker', 'ojs/ojbutton', 'ojs/ojknockout', 'ojs/ojinputtext', 'ojs/ojlabel'],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;
    function EditVacationViewModel() {
      var self = this;

      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      this.title = userId;
      this.ferId = "";
      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        var urlString = window.location.href;
        var url = new URL(urlString);
        var values = url.searchParams.get("root").split("/");
        this.ferId = values[1]; // ID of the vacation is sent through the url
        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFerie/" + values[1];

        // Get data for the specifc vacation from the database
        var xhttp = new XMLHttpRequest();
        xhttp.open("GET", url, true);
        xhttp.send();
        var dateTo = document.getElementById("dateTo");
        var dateFrom = document.getElementById("dateFrom");
        var other = document.getElementById("other");

        // Fill in the fields with the current data
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
             // Typical action to be performed when the document is ready:
             var JSONData = JSON.parse(this.responseText);
             console.log(JSONData);
             other.value = JSONData.fer_kommentar.toString();
             var jsonDateFrom = new Date(JSONData.fer_fra_dato);
             var jsonDateTo = new Date(JSONData.fer_til_dato);

             // Format dates into ISO 8601 format
             var dateFromString = jsonDateFrom.getFullYear() + '-'
                                + ('0' + (jsonDateFrom.getMonth()+1)).slice(-2) + '-'
                                + ('0' + jsonDateFrom.getDate()).slice(-2);

             var dateToString = + jsonDateTo.getFullYear() + '-'
                              + ('0' + (jsonDateTo.getMonth()+1)).slice(-2) + '-'
                              + ('0' + jsonDateTo.getDate()).slice(-2);

             dateTo.value = dateToString;
             dateFrom.value = dateFromString;
          }
        };
      };
      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Function for saving edited data to the database
      self.saveData = function() {
        var data = {};
        data.fer_dato_fra = (document.getElementById("dateFrom").value).substring(0, 10);
        data.fer_dato_til = (document.getElementById("dateTo").value).substring(0, 10);
        data.fer_kommentar = document.getElementById("other").value;

        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFerie/" + self.ferId;
        var xhttp = new XMLHttpRequest();
        xhttp.open("PUT", url, true);
        xhttp.setRequestHeader('Content-type', 'application/json');
        var json = JSON.stringify(data);
        xhttp.send(json);
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            rootViewModel.showSaveSuccess("Endringer er nå lagret");
            router.go('/unavailability');
          }
        }
      };

      // Function for deleting data from the database
      self.deletePost = function() {
        var url = rootViewModel.baseUrl() + "utilgjengelighet/redigerFerie/" + self.ferId;
        var xhttp = new XMLHttpRequest();
        xhttp.open("DELETE", url, true);
        xhttp.send();
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            rootViewModel.showDeleteSuccess("Du er nå tilgjengelig på dette tidspunktet");
            router.go('/unavailability');
          }
        }
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new EditVacationViewModel();
  }
);
