/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your myInterests ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojarraydataprovider', 'ojs/ojlistview', 'ojs/ojbutton', 'ojs/ojrouter'],
 function(oj, ko, $, app, moduleUtils, ArrayDataProvider) {
  
    function MyInterestsViewModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      self.rootViewModel;           // Initiate variable.
      self.data = [];
      self.datasource = ko.observableArray(self.data);
      
      self.getInterests = function(url) {
        $.ajax({
          dataType: "json",
          url: url,
          success: function(interests) {
            $.each(interests.items, function() {
              self.data.push({
                  id: this.id, 
                  item: this.navn,
                  description: this.beskrivelse,
                  status: this.status,
                  category: this.kategori
              });
    
            });
            self.datasource(self.data);     // Insert into observable array.
          },
          error: function() { self.rootViewModel.showGeneralError(); }
        });
      }

      self.datasource(self.data);

    self.dataProvider = new ArrayDataProvider(self.datasource, {'keyAttributes': 'id'});

    // Delete entry when clicking delete button:
    self.deleteButtonClicked = function(event){
      var itemId = event.currentTarget.id.split("Button")[1];    // Get item/button id.

      $.ajax({
        url: self.rootViewModel.baseUrl() + "frivillig/" + self.rootViewModel.userId() + "/interesser/" + itemId,
        type: "DELETE",
        success: function(result){
          self.datasource.remove(function(item)
          {
              return (item.id == itemId);
          });
        }
    });
    }

    self.addButtonClicked = function(){
      oj.Router.rootInstance.go('addInterests');
    }

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here. 
       * This method might be called multiple times - after the View is created 
       * and inserted into the DOM and after the View is reconnected 
       * after being disconnected.
       */
      self.connected = function() {
        self.rootViewModel = ko.dataFor(document.getElementById('globalBody'));     // Refresh every time we go here.
        
        //If userId is undefined, go back to login-page:
        if (self.rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        // Zero out data before refresh/adding data to list:
        self.data = [];
        self.getInterests(self.rootViewModel.baseUrl() + "frivillig/" + self.rootViewModel.userId() + "/interesser");
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    
    return new MyInterestsViewModel();
  }
);
