/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojselectcombobox', 'ojs/ojinputnumber', 'ojs/ojselectcombobox', 'ojs/ojbutton', 'ojs/ojknockout', 'ojs/ojinputtext', 'ojs/ojlabel', "ojs/ojdatetimepicker"],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;
    function EditUnavailabilityViewModel() {
      var self = this
      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();

      self.timeFrom = ko.observable(oj.IntlConverterUtils.dateToLocalIso(new Date(2000, 0, 1)));
      self.timeTo = ko.observable(oj.IntlConverterUtils.dateToLocalIso(new Date(2000, 0, 1)));

      this.title = userId;
      this.tghId = "";
      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Function for inserting a new post into the database
      self.saveData = function() {

        var data = {};
        data.tgh_dag = document.getElementById("basicSelect").value;
        data.tgh_kl_fra = timeToDecimal(self.timeFrom());
        data.tgh_kl_til = timeToDecimal(self.timeTo());
        data.tgh_merknad = document.getElementById("other").value;

        var url = rootViewModel.baseUrl() + "utilgjengelighet/nyFast/" + userId;
        var xhttp = new XMLHttpRequest();
        xhttp.open("POST", url, true);
        xhttp.setRequestHeader('Content-type', 'application/json');
        var json = JSON.stringify(data);
        xhttp.send(json);

        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            rootViewModel.showSaveSuccess("Du er nå utilgjengelig på dette tidspunktet");
            router.go('/unavailability');
          }
        }
      };

    }
    // Converts the ISOtime to decimal with four digits
    // Example: T14:52:00 -> 1452
    function timeToDecimal(time) {
      var hoursMinutes = time.split(/[T.:]/);
      var hours = hoursMinutes[1];
      var minutes = hoursMinutes[2];
      return hours + minutes;
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new EditUnavailabilityViewModel();
  }
);
