/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your profile ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojinputtext'],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;

    function ProfileViewModel(viewParams) {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */

      self.connected = function() {
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        console.log(rootViewModel.userId._latestValue);
        // Gets all userdata aout the user that is logged in
        var xhttp = new XMLHttpRequest();
        var url = rootViewModel.baseUrl() + "oppdater/hent/" + rootViewModel.userId._latestValue;
        xhttp.open("GET", url, true);
        xhttp.send();
        xhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {
               // Typical action to be performed when the document is ready:
               var JSONData = JSON.parse(this.responseText);
               document.getElementById("emailTxt").innerHTML = "Email: " + JSONData.bru_epost;
               document.getElementById("nameTxt").innerHTML = "Navn: " + JSONData.bru_fornavn + " " + JSONData.bru_etternavn;
               if (JSONData.bru_mobiltelefon) {
                 document.getElementById("phoneTxt").innerHTML = "Mobil: " + JSONData.bru_mobiltelefon;
               } else if (JSONData.bru_telefonnr) {
                 document.getElementById("phoneTxt").innerHTML = "Tlf: " + JSONData.bru_telefonnr;
               } else {
                 document.getElementById("phoneTxt").innerHTML = "BrukerID: " + JSONData.bru_brukerid;
               }
            }
        };
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Functions that routes the user to different pages when a click has been detected
      self.handleBack = function() {
        router.go('/incidents');
      };
      self.notification = function() {
        router.go('/settings');

      };
      self.interests = function() {
        router.go('/myInterests');
      };
      self.edit = function(event) {
        // Sends the datatype and value as parameters in the url
        var text = event.currentTarget.innerText;
        var value = text.split(": ");

        router.go('/editPerson/' + value[0] + '/' + value[1]);
      };
      self.editPassword = function(event) {
        router.go('/editPassword');
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new ProfileViewModel();
  }
);
