/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */

/*
 * Your dashboard RegisterHoursListModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojarraydataprovider', 'moment', 'ojs/ojlistview', 'ojs/ojrouter', 'ojs/ojbutton'],
 function(oj, ko, $, app, moduleUtils, ArrayDataProvider, moment) {
  
    function RegisterHoursListModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      self.data = [];
      self.datasource = ko.observableArray(self.data);
      self.nextUrl = ko.observableArray();
      
      self.getServices = function(url) {
          console.log("Started to get services")

          $.ajax({
            dataType: "json",
            url: url,
            success: function(services) {
              $.each(services.items, function() {
                let date = moment(this.dato_tid);
                let editedTime;
                let endDate = moment(date).add(this.original_varighet, 'hours');
                let endDateFinal = moment(date).add(this.endret_varighet, 'hours');
                // If a time is changed, write new time, if not, write empty line.
                if (this.timer_endret == "JA") {
                  // Convert duration to a time:
                  editedTime = "(Ny sluttid: " +endDateFinal.format('HH:mm') + ")";
                }
                else {
                  editedTime = ""
                }
                self.data.push({
                    id: this.id, 
                    item: this.navn,
                    date: date.format('DD.MM.YYYY'),
                    startTime: date.format('HH:mm'),
                    endTime: endDate.format('HH:mm'),
                    editedEnd: editedTime
                });
                
                // If we has more to load if we want, enable a "load more"-button and save url to load next:
                let loadMoreButton = document.getElementById('loadMoreButton');
                if (services.hasMore) {
                  for (var i = 0; i < services.links.length; i++) {
                    if (services.links[i].rel == "next") {
                      self.nextUrl = services.links[i].href;
                    }
                  }
                  loadMoreButton.disabled = false;
                }
                else {                                    // If has not more to load, disable button.
                  loadMoreButton.disabled = true;
                }
            });
            self.datasource(self.data);     // Insert into observable array.
        },
            error: function() { self.rootViewModel.showGeneralError(); }
          });
      }

      self.datasource(self.data);

    self.dataProvider = new ArrayDataProvider(self.datasource, {'keyAttributes': 'id'});

    self.selectedItem = function(event) {
      let itemId = event.detail.value;
      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      rootViewModel.serviceId(itemId);
      oj.Router.rootInstance.go('editRegisteredHours');
    }

    self.loadMoreButtonClicked = function() {
      document.getElementById('loadMoreButton').disabled = true;      // Disable until loaded more.
      self.getServices(self.nextUrl)
    }

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here. 
       * This method might be called multiple times - after the View is created 
       * and inserted into the DOM and after the View is reconnected 
       * after being disconnected.
       */
      self.connected = function() {
        // Get logged-in id:
        var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        document.getElementById('loadMoreButton').disabled = true;      // Disable until loaded some entries.

        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        
        // Zero out data before refresh/adding data to list:
        self.data = [];
        self.getServices(rootViewModel.baseUrl() + "frivillig/"+ rootViewModel.userId() + "/avtaler/eldre");
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new RegisterHoursListModel();
  }
);
