/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'moment',
        'ojs/ojbutton', 'ojs/ojradioset', 'ojs/ojslider', 'ojs/ojlabel'],
 function(oj, ko, $, app, moduleUtils, moment) {

    function ServiceInfoViewModel() {
      var self = this;
      this.contractId = 0;
      this.serviceLength = 0;
      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      var volId = getVolunteerId();
      var fromDB = false;
      this.title = userId;

      this.activityItems = [
            {id: 'Ikke interessert'},
            {id: 'Usikker'},
            {id: 'Interessert'}
        ];

      this.missionItems = [
            {id: 'Kan ikke'},
            {id: 'Usikker'},
            {id: 'Aksepter'}
        ];

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
       function getUserData(tje_id) {
         var url = rootViewModel.baseUrl() + "tjenester/meldinteresse/";
         url += volId + "/" + tje_id;
         var xhttp = new XMLHttpRequest();
         xhttp.open("GET", url, true);
         xhttp.send();

         xhttp.onreadystatechange = function() {
           console.log("GetUserData: " + this.readyState + " | " + this.status);
           if (this.readyState == 4 && this.status == 200) {
             var JSONData = JSON.parse(this.responseText);
             console.log(JSONData);
             if (JSONData.count == 0) { self.contractId = 0;}
             else {
               fromDB = true;
               self.contractId = JSONData.items[0].fav_id;
               var checkbar;
               if (self.serviceState == "mission") {
                 checkbar = document.getElementById("missionStatus");
               }
               else {
                 checkbar = document.getElementById("activityStatus");
               }
               checkbar.value = JSONData.items[0].fav_status;
             }
           }
         }
       }

      self.connected = function() {
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        
        var urlString = window.location.href;
        var url = new URL(urlString);
        var values = url.searchParams.get("root").split("/");
        this.tjeId = values[1]; // Id of the service is sent through the url
        console.log(values[1])
        var url = rootViewModel.baseUrl() + "tjenester/oppdrag/" + values[1];

        //Get all data from the specific servie
        var xhttp = new XMLHttpRequest();
        xhttp.open("GET", url, true);
        xhttp.send();
        //  Fill in the fields with the current data
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            var JSONData = JSON.parse(this.responseText);
            console.log(JSONData);
            var name = document.getElementById("1");
            var description = document.getElementById("2");
            var location = document.getElementById("3");
            var date = document.getElementById("4");
            var time = document.getElementById("5")
            var contact = document.getElementById("6");

            name.innerHTML = JSONData.tje_navn;
            description.innerHTML = JSONData.tje_beskrivelse;
            location.innerHTML = JSONData.tje_sted;
            if (JSONData.tje_kontaktperson != null) {
              contact.innerHTML = "Kontaktperson: " + JSONData.tje_kontaktperson + "<br>" + JSONData.tje_kontaktperson_epost;
            }
            self.serviceLength = JSONData.tje_varighet;

            var dateTime = moment(JSONData.tje_dato_tid);
            var dateFormat = dateTime.format("DD.MM.YYYY");
            date.innerHTML = dateFormat;

            var timeFormat = dateTime.format("HH:mm");

            var timeEnd = moment(dateTime).add(JSONData.tje_varighet, 'hours');
            date.innerHTML +=  "<br>Kl. " + timeFormat + " - kl." + timeEnd.format("HH:mm");

            switch (JSONData.tjt_kategori) {
              case "Aktivitet":
                var missionStatus = document.getElementById("missionStatus");
                missionStatus.parentNode.removeChild(missionStatus);
                self.serviceState = "activity";
                break;
              case "Oppdrag":
                var activityStatus = document.getElementById("activityStatus");
                activityStatus.parentNode.removeChild(activityStatus);
                self.serviceState = "mission";
                break;
            }
            userData = getUserData(values[1]);
          }
        };
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      self.interest = function(event) {
        var data = {};
        var url = rootViewModel.baseUrl() + "tjenester/meldinteresse";
        url += "/" + volId + "/" + self.tjeId;
        console.log(self.contractId);
        var xhttp = new XMLHttpRequest();
        if (self.contractId == 0) {
          xhttp.open("POST", url, true);
          data.status = event.detail.value;
          data.tje_varighet = self.serviceLength;
          console.log("Url: " + url);
        }
        else {
          xhttp.open("PUT", url, true);
          data.contract_id = self.contractId;
          data.status = event.detail.value;
        }
        xhttp.setRequestHeader('Content-type', 'application/json');

        var json = JSON.stringify(data);
        console.log(json);
        xhttp.send(json);
        xhttp.onreadystatechange = function() {
          console.log("SAAAVE:" + this.readyState + " | " + this.status);
          if (this.readyState == 4 && this.status == 200) {
            if (self.contractId == 0) {
                var JSONData = JSON.parse(this.responseText);
                self.contractId = JSONData.favId;
            }
            if (fromDB) fromDB = false;
            else rootViewModel.showSaveSuccess("Du har nå oppdatert statusen din på dette arrangementet");
            console.log("Complete");
          }
        }
      };

      function getVolunteerId() {
        var url = rootViewModel.baseUrl() + "bruker/frivillig/" + userId;
        var xhttp = new XMLHttpRequest();
        xhttp.open("GET", url, true);
        xhttp.send();
        //  Fill in the fields with the current data
        xhttp.onreadystatechange = function() {
          if (this.readyState == 4 && this.status == 200) {
            var JSONData = JSON.parse(this.responseText);
            console.log(JSONData.items[0].frv_id);
            volId = JSONData.items[0].frv_id;
          }
        }
      }
    }
    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new ServiceInfoViewModel();
  }
);
