/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'moment',
      'ojs/ojarraydataprovider', 'ojs/ojknockout', 'ojs/ojlistview', 'ojs/ojbutton', 'ojs/ojinputtext',
      'ojs/ojlabel', 'ojs/ojnavigationlist'],
 function(oj, ko, $, app, moduleUtils, moment) {

    function ServicesViewModel() {
      var self = this;
      var router = oj.Router.rootInstance;

      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      var volId;
      var myActivityElements = [];
      var serviceIdList = [];
      var animationCounter = 0;
      this.selectedState = ko.observable("missions");

      this.title = userId;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      function getData(selectedState) {
        animationCounter = 0;
        var arrLength = self.allItems().length;
        for(var i = 0; i < arrLength; i++) {
          self.allItems.pop(i);
        }

        serviceIdList = [];
        getVolunteerId();

        var otherTab1;
        var otherTab2
        switch (selectedState) {
          case "activities": otherTab1 = "missions"; otherTab2 = "myActivities"; break;
          case "missions": otherTab1 = "activities"; otherTab2 = "myActivities"; break;
          case "myActivities": otherTab1 = "missions"; otherTab2 = "activities"; break;
        }
        var otherTabElement1 = document.getElementById(otherTab1);
        otherTabElement1.classList.toggle("oj-disabled");
        var otherTabElement2 = document.getElementById(otherTab2);
        otherTabElement2.classList.toggle("oj-disabled");

        $.ajax({
          url: rootViewModel.baseUrl() + "tjenester/aktiviteter",
          type: "GET",
          success: function(data){
            var JSONData = data;
            var currItems = 0;
            for (var i = 0; i < JSONData.items.length; i++) {
              var dateTime = moment(JSONData.items[i].tje_dato_tid);
              var dateFormat = dateTime.format("DD.MM.YYYY");
              var timeFormat = dateTime.format("HH:mm");

              var timeEnd = moment(dateTime).add(JSONData.items[i].tje_varighet, 'hours');
              var title = JSONData.items[i].tje_navn;
              var info = JSONData.items[i].geo_navn + " - " + dateFormat + " " + timeFormat + " - " + timeEnd.format("HH:mm");
              if (selectedState == "activities" && JSONData.items[i].tjt_kategori == "Aktivitet" && currItems < 10 ) {
                self.allItems.push({"id": JSONData.items[i].tje_id, "title": title, "info": info});
                serviceIdList[currItems] = JSONData.items[i].tje_id;
                currItems++;
              } else if (selectedState == "missions" && JSONData.items[i].tjt_kategori == "Oppdrag" && currItems < 10) {
                self.allItems.push({"id": JSONData.items[i].tje_id, "title": title, "info": info});
                serviceIdList[currItems] = JSONData.items[i].tje_id;
                currItems++;
              } else if (selectedState == "myActivities" && hasInterest(JSONData.items[i].tje_id) && currItems < 10) {
                self.allItems.push({"id": JSONData.items[i].tje_id, "title": title, "info": info});
                serviceIdList[currItems] = JSONData.items[i].tje_id;
                currItems++;
              }
            }
          },
          complete: function() {
            otherTabElement1.classList.toggle("oj-disabled");
            otherTabElement2.classList.toggle("oj-disabled");
            var tabBar = document.getElementById("tabBar");
            tabBar.refresh();

          }
        });
      }

      self.connected = function() {
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        
        myActivityElements = [];
        this.itemToAdd = ko.observable("");
        this.allItems = ko.observableArray([]);
        this.selectedItems = ko.observableArray([]);
        this.dataProvider = new oj.ArrayDataProvider(this.allItems, {keyAttributes: 'id'});
        getData(self.selectedState());
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      self.itemSelected = function() {
         router.go('/serviceInfo/' + self.selectedItems()[0]);
      };

      self.changeView = function () {
        getData(self.selectedState());
      };

      function getVolunteerId() {
        $.ajax({
          url:  rootViewModel.baseUrl() + "bruker/frivillig/" + userId,
          type: "GET",
          success: function(data) {
            volId = data.items[0].frv_id;
            $.ajax({
              url: rootViewModel.baseUrl() + "tjenester/mineTjenester/" + volId,
              type: "GET",
              success: function(data) {
                myActivityElements = data.items;
              },
            });
          }
        });
      }

      function hasInterest(serviceId) {
        var hit = false;
        for(var i = 0; i < myActivityElements.length; i++) {
          if (serviceId == myActivityElements[i].fav_tje_id) {
            if (myActivityElements[i].fav_status == "Aksepter" || myActivityElements[i].fav_status == "Interessert" || myActivityElements[i].fav_status == "Godkjent") {
              hit = true;
            }
          }
        }
        return hit;
      }

      self.colorpicker = function(event) {

        animationCounter++;
        if (animationCounter >= serviceIdList.length) {
          console.log(serviceIdList.length);
          console.log("Hello");
          console.log(event);

          var lis = document.getElementById("listViewContainer").getElementsByTagName("li");
          for (var j = 0; j < serviceIdList.length; j++) {
            var element = lis[j];
            for(var i = 0; i < myActivityElements.length; i++) {
              if (serviceIdList[j] == myActivityElements[i].fav_tje_id) {
               switch (myActivityElements[i].fav_status) {
                 case "Aksepter": element.style = "background-color: #a8ff8e"; break;
                 case "Godkjent": element.style = "background-color: #a8ff8e"; break;
                 case "Kan ikke": element.style = "background-color: #ff7777";; break;
                 case "Ikke interessert": element.style = "background-color: #ff7777"; break;
                 case "Interessert": element.style = "background-color: #ffee7f"; break;
                 default: element.style = "background-color: #ffffff"; break;
               }
             }
            }
          }
        }
      };
    }



    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new ServicesViewModel();
  }
);
