/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your incidents ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'ojs/ojcheckboxset', 'ojs/ojbutton', 'ojs/ojlabel'],
 function(oj, ko, $, app, moduleUtils) {

   var router = oj.Router.rootInstance;
   var rootViewModel;

    function IncidentsViewModel() {
      var self = this;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        rootViewModel = ko.dataFor(document.getElementById('globalBody'));
        //If userId is undefined, go back to login-page:
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        //Gets the current notificationsettings from the database
        var url = rootViewModel.baseUrl() + "innstilling/" + rootViewModel.userId._latestValue;
        var xhttp = new XMLHttpRequest();
        xhttp.open("GET", url, true);
        xhttp.send();
        xhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {
               // Goes through the data and makes the corresponding checkboxes checked
               var JSONData = JSON.parse(this.responseText);
               console.log(JSONData.items[0].frv_varsling);
               var notiData = JSONData.items[0].frv_varsling.split(" og ");
               var selectedNoti = [];
               var counter = 0;
               for (var i = 0; i < notiData.length; i++) {
                 switch (notiData[i]) {
                   case "SMS": selectedNoti[counter++] = "sms"; break;
                   case "epost": selectedNoti[counter++] = "epost"; break;
                 }
               }
               console.log(selectedNoti)
               document.getElementById("options").value = selectedNoti;
            }
        };
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      // Function for saving updated notificationsettings
      self.saveData = function() {
        var data = {};
        var dataString = "";
        var url = rootViewModel.baseUrl() + "innstilling/" + rootViewModel.userId._latestValue;
        var xhttp = new XMLHttpRequest();
        for (var i = 0; i < document.getElementById("options").value.length; i++) {
          switch (document.getElementById("options").value[i]) {
            case "sms": dataString += "SMS"; break;
            case "epost": dataString += "epost"; break;
          }
          if (i < document.getElementById("options").value.length - 1) dataString += " og ";
        }
        data.varsling = dataString;
        xhttp.open("PUT", url, true);
        xhttp.setRequestHeader('Content-type', 'application/json');
        var json = JSON.stringify(data);
        xhttp.send(json);

        xhttp.onreadystatechange = function() {
          if (xhttp.readyState == XMLHttpRequest.DONE) {
            rootViewModel.showSaveSuccess("Endringer er nå lagret");
            router.go('/profile');
          }
        }
      };
    }


    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new IncidentsViewModel();
  }
);
