/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */

/*
 * Your showing of id ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils', 'moment', 'ojs/ojavatar', 'ojs/ojanimation', 'ojs/ojvalidation','ojs/ojlabel', 'ojs/ojbutton'],
 function(oj, ko, $, app, moduleUtils, moment) {

    function ShowIdViewModel() {
      var self = this;

      self.rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = self.rootViewModel.userId();

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })


      /**
       * Show animation to validate that it is not just an image.
       */
      self.validateButtonClicked = function() {
        var elem = document.getElementById('avatarDiv');
        // Animate the element
        oj.AnimationUtils['flipOut'](elem, {'flipTarget': 'children',
                                            'persist': 'all',
                                            'startAngle': '0deg',
                                            'endAngle': '360deg'});
      }

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */
      self.connected = function() {
        //If userId is undefined, go back to login-page (not sure if we shall have this check).
        if (self.rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }
        
        // Get updated id-card:
        self.data = {};
        self.datasource = ko.observable(self.data);
        let url = self.rootViewModel.baseUrl() + "frivillig/" + self.rootViewModel.userId() + "/idkort";
        $.ajax({
          dataType: "json",
          url: url,
          success: function(person) {
            self.data = {
              firstname: person.fornavn,
              lastname: person.etternavn,
              initials: oj.IntlConverterUtils.getInitials(person.fornavn,person.etternavn),
              occupation: person.yrke,
              orgnr: person.orgnr,
              organization: person.org_navn,
              validString: person.frv_status + " per " + moment().format('DD.MM.YYYY')
            };
            self.datasource(self.data);
          },
          error: function() { self.rootViewModel.showGeneralError(); }
        });

        self.datasource(self.data);
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };
    }

    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new ShowIdViewModel();
  }
);
