/**
 * @license
 * Copyright (c) 2014, 2019, Oracle and/or its affiliates.
 * The Universal Permissive License (UPL), Version 1.0
 */
/*
 * Your dashboard ViewModel code goes here
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'appController', 'ojs/ojmodule-element-utils',
      'ojs/ojarraydataprovider', 'ojs/ojknockout', 'ojs/ojlistview', 'ojs/ojbutton', 'ojs/ojinputtext',
      'ojs/ojlabel', 'ojs/ojnavigationlist'],
 function(oj, ko, $, app, moduleUtils) {

    function UnavailabilityViewModel() {
      var self = this;
      var rootViewModel = ko.dataFor(document.getElementById('globalBody'));
      var userId = rootViewModel.userId();
      var router = oj.Router.rootInstance;

      this.selectedState = ko.observable("regular");

      this.title = userId;

      // Header Config
      self.headerConfig = ko.observable({'view':[], 'viewModel':null});
      moduleUtils.createView({'viewPath':'views/header.html'}).then(function(view) {
        self.headerConfig({'view':view, 'viewModel':new app.getHeaderModel()})
      })

      // Below are a set of the ViewModel methods invoked by the oj-module component.
      // Please reference the oj-module jsDoc for additional information.

      /**
       * Optional ViewModel method invoked after the View is inserted into the
       * document DOM.  The application can put logic that requires the DOM being
       * attached here.
       * This method might be called multiple times - after the View is created
       * and inserted into the DOM and after the View is reconnected
       * after being disconnected.
       */

      function getData(selectedState) {
        console.log(selectedState);
        var otherTab = ((selectedState == "regular") ? "vacation" : "regular");
        console.log("Other: " + otherTab);
        var otherTabElement = document.getElementById(otherTab);
        otherTabElement.classList.toggle("oj-disabled");
        var arrLength = self.allItems().length;
        for(var i = 0; i < arrLength; i++) {
          self.allItems.pop(i);
        }

        var url = rootViewModel.baseUrl() + "utilgjengelighet/";
        if (selectedState == "regular") {
          url += "fast/";
          url += userId;
          var xhttp = new XMLHttpRequest();
          xhttp.open("GET", url, true);
          xhttp.send();

          xhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {
               // Typical action to be performed when the document is ready:
               var JSONData = JSON.parse(this.responseText);
               for (var i = 0; i < JSONData.items.length; i++) {
                 var data = "";
                 switch (JSONData.items[i].tgh_dag) {
                   case 1: data += "Mandag "; break;
                   case 2: data += "Tirsdag"; break;
                   case 3: data += "Onsdag "; break;
                   case 4: data += "Torsdag"; break;
                   case 5: data += "Fredag "; break;
                   case 6: data += "Lørdag "; break;
                   case 7: data += "Søndag "; break;
                 }
                 // Converts the time from the database into time format
                 var from = JSONData.items[i].tgh_kl_fra;
                 var to = JSONData.items[i].tgh_kl_til;

                 data += " (" +  decimalToTime(JSONData.items[i].tgh_kl_fra) + " - " +
                 decimalToTime(JSONData.items[i].tgh_kl_til) + "): " + JSONData.items[i].tgh_merknad;
                 console.log(JSONData.items[i]);

                 // The following IF statements adds 0 in front of the timeout
                 // if the time is only one digit

                 // Checks if both times are one digit
                 /*if (decimalToTime(JSONData.items[i].tgh_kl_fra) < 10 && decimalToTime(JSONData.items[i].tgh_kl_til) < 10) {
                   data += " (0" +  decimalToTime(JSONData.items[i].tgh_kl_fra) + " - 0" +
                   decimalToTime(JSONData.items[i].tgh_kl_til) + "): " + JSONData.items[i].tgh_merknad;
                 }
                 // Checks if only the time from is one digit
                 else if (decimalToTime(JSONData.items[i].tgh_kl_fra) < 10 && decimalToTime(JSONData.items[i].tgh_kl_til) >= 10) {
                   data += " (0" +  decimalToTime(JSONData.items[i].tgh_kl_fra) + " - " +
                   decimalToTime(JSONData.items[i].tgh_kl_til) + "): " + JSONData.items[i].tgh_merknad;
                 }
                 // Checks if only the time to is one digit
                 else if (decimalToTime(JSONData.items[i].tgh_kl_fra) >= 10 && decimalToTime(JSONData.items[i].tgh_kl_til) < 10) {
                   data += " (" +  decimalToTime(JSONData.items[i].tgh_kl_fra) + " - 0" +
                   decimalToTime(JSONData.items[i].tgh_kl_til) + "): " + JSONData.items[i].tgh_merknad;
                 }
                 // If none are met, it means both are two digits, nothing is done
                 else {
                   data += " (" +  decimalToTime(JSONData.items[i].tgh_kl_fra) + " - " +
                   decimalToTime(JSONData.items[i].tgh_kl_til) + "): " + JSONData.items[i].tgh_merknad;
                 }*/

                 self.allItems.push({"id": JSONData.items[i].tgh_id, "item": data});
              }
              otherTabElement.classList.toggle("oj-disabled");
              var tabBar = document.getElementById("tabBar");
              tabBar.refresh();
            }
          }
        } else if (selectedState == "vacation") {
          url += "ferie/";
          url += userId;
          var xhttp = new XMLHttpRequest();
          xhttp.open("GET", url, true);
          xhttp.send();

          xhttp.onreadystatechange = function() {
            if (this.readyState == 4 && this.status == 200) {
               // Typical action to be performed when the document is ready:
               var JSONData = JSON.parse(this.responseText);
               for (var i = 0; i < JSONData.items.length; i++) {
                 console.log(JSONData.items[i]);
                 var data = "";
                 var fromDate = new Date(JSONData.items[i].fer_fra_dato);
                 var fromDateFormat = fromDate.getDate() + "/" + (fromDate.getMonth() + 1) + "/" + fromDate.getFullYear();
                 var toDate = new Date(JSONData.items[i].fer_til_dato);
                 var toDateFormat = toDate.getDate() + "/" + (toDate.getMonth() + 1) + "/" + toDate.getFullYear();
                 data += fromDateFormat + " - " + toDateFormat  + ": " + JSONData.items[i].fer_kommentar;
                 self.allItems.push({"id": JSONData.items[i].fer_id, "item": data, });
              }
              otherTabElement.classList.toggle("oj-disabled");
              var tabBar = document.getElementById("tabBar");
              tabBar.refresh();
            }
          }
        }
      }

      self.connected = function() {
        //If userId is undefined, go back to login-page (not sure if we shall have this check).
        if (rootViewModel.userId() == undefined) {
          oj.Router.rootInstance.go('login');
          return
        }

        this.itemToAdd = ko.observable("");
        this.allItems = ko.observableArray([]);
        this.selectedItems = ko.observableArray([]);
        this.dataProvider = new oj.ArrayDataProvider(this.allItems, {keyAttributes: 'id'});

        getData(self.selectedState());
      };

      /**
       * Optional ViewModel method invoked after the View is disconnected from the DOM.
       */
      self.disconnected = function() {
        // Implement if needed
      };

      /**
       * Optional ViewModel method invoked after transition to the new View is complete.
       * That includes any possible animation between the old and the new View.
       */
      self.transitionCompleted = function() {
        // Implement if needed
      };

      self.addButtonClicked = function() {
        switch (self.selectedState()) {
          case "vacation": router.go('/newVacation'); break;
          case "regular": router.go('/newUnavailability'); break;
        }
      };

      self.itemSelected = function() {
        switch (self.selectedState()) {
          case "vacation": router.go('/editVacation/' + self.selectedItems()[0]); break;
          case "regular": router.go('/editUnavailability/' + self.selectedItems()[0]); break;
        }
      };

      self.changeView = function () {
        getData(self.selectedState());
      }
    }

    // Converts the time from the database to a visual time format
    // Example: 2351 -> 23:51
    function decimalToTime(time) {
      var minutes = time % 100;
      var hours = (time - minutes) / 100;
      console.log("Timer: " + hours + " | Minutes: " + minutes);
      if (hours < 10 && minutes < 10) return "0" + hours + ":0" + minutes;
      else if (hours < 10 && minutes >= 10) return "0" + hours + ":" + minutes;
      else if (hours >= 10 && minutes < 10) return hours + ":0" + minutes;
      else return hours + ":" + minutes;
      }
    /*
     * Returns a constructor for the ViewModel so that the ViewModel is constructed
     * each time the view is displayed.  Return an instance of the ViewModel if
     * only one instance of the ViewModel is needed.
     */
    return new UnavailabilityViewModel();
  }
);
